/**
  *  \file game/utilparse.h
  *  \brief UTIL.DAT parsing for game information
  *
  *  (c) 2001-2021 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_UTILPARSE_H
#define PCC_V2_GAME_UTILPARSE_H

#include <vector>
#include "game/parser.h"
#include "cpluslib/ptrvector.h"
#include "game/utilread.h"
#include "game/utildata.h"

class GUniverse;
class GGameTurn;

/** UTIL.DAT Message Parser. Interprets UTIL.DAT records that yield
    message-style information. */
class GUtilMessageParser {
    const GUniverse& univ;
    std::vector<TUtil7Battle> battle_results;
    std::vector<bool> destroyed_ships;

    void killShip(int sid);
 public:
    GUtilMessageParser(const GUniverse& univ);

    void parseRecord(int rec_id, std::size_t size, const char* data,
                     ptr_vector<GMessageInformation>& info_out);

    void finish(GGameTurn& trn);
};

/** UTIL.DAT Message Parser. Interprets all UTIL.DAT records that yield
    some information for a universe object. */
class GUtilParser : public GUtilReader {
    GGameTurn& trn;
    GUtilMessageParser parser;
    int player;
 public:
    GUtilParser(GGameTurn& trn, int player);
    ~GUtilParser();

    bool process(int rec_id, std::size_t size, const char data[]);
    void failure(Stream& s);

    void finish();
};

#endif
